clear
e = 1.602e-19;% electron charge
% % % % % 
%Inputs
% % % % % 
N = 20;   % Truncation order: Conevergence parameter. 
L = 8e-6; % Period
W = 0.5*L; % Ribbons width
Ef = 0.6*e; % Fermi energy
tau = .25e-12; % Relaxation time
lambda = 300e-6/L; % Free space wavelength normalized to the grating period
theta = 0*pi/180; % incident angle
n1 = 3^0.5;   % Refractive index of incident medium
n3 = 4^0.5; % Refractive index of transmission medium
% -----------------------------------
% Defining grating region parameteres
% -----------------------------------
% Transition (discontinuity) points in profile of refractive index are
% defined by 'xl'. You must set first and last element of 'xl' to 0 and 1,
% repectively. It means that the first transition occurs at the begining of
% the grating period. 
xl = [0 W/L 1]; 

% The conductivity between transitions is determined by 'Gl'. 'nl(m)' 
% is the conductivity of the region between xl(m) and xl(m+1).
G0 = conductivity(Ef,tau,lambda*L);
Gl = [G0 0];
% -------------------
% Initial computaions
% -------------------
k0=2*pi/lambda;
omega = k0*(3e8);

mue0 = 4e-7*pi;
mue1 = mue0;
mue3 = mue0;

k1=k0*n1;
k3=k0*n3;
kx=zeros(2*N+1,1);
kz1 = kx;
kz3= kx;
kx(1:2*N+1)=2*pi*(n1*sin(theta)/lambda-(-N:N));
kz1(1:2*N+1)=sqrt(k1^2-kx(1:2*N+1).^2);
kz3(1:2*N+1)=sqrt(k3^2-kx(1:2*N+1).^2);
for j=1:2*N+1
    if -real(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    if imag(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    
    if -real(kz3(j))>1e-12
        kz3(j)=-kz3(j);
    end
    if imag(kz3(j))>1e-12
        kz3(j)=-kz3(j);
    end
end
Kx=diag(kx);
Kz1=diag(kz1);
Kz3=diag(kz3);
I=eye(2*N+1);
% --------------------

% ------------------------------------------
% Construction of required toeplitz matrices
% ------------------------------------------
lxl = length(xl);
gmn=fastftptrain(xl(2:(lxl-1)),Gl,2*N);
Gmn=toeplitz(gmn(2*N+1:4*N+1),gmn(2*N+1:-1:1));
% ------------------------------------------
% --------------------
% S matrix calculation
% --------------------
eta1 = diag((omega*mue1)./kz1);
eta3 = diag((omega*mue3)./kz3);
etas = diag((kz3*mue1)./(kz1*mue3));
zays = (Gmn*eta1);

M11 = (I*1/2)*(I+etas+zays);
M21 = (I*1/2)*(I-etas-zays);
% ------------------

incident=zeros(2*N+1,1);
incident(N+1)=1;

% Reflection and transmission coefficient in x coordinate
Tte=M11\incident;
Rte=M21*(M11\incident);

% Diffraction efficiencies
DE1=zeros(1,2*N+1);
DE3=zeros(1,2*N+1);
for j=1:2*N+1
    DE1(j)=(abs(Rte(j))^2).*real(kz1(j)/k1/cos(theta));
    DE3(j)=(abs(Tte(j))^2).*real(kz3(j)/(k1*cos(theta)));
end
1-sum(DE1)-sum(DE3)